@@ -1,5 +1,4 @@
 # with input in the DataFrame format
-# handle categorical values
 # update metadata of the dataset
 
 import rpi_feature_selection_toolbox
@@ -8,7 +7,7 @@ import typing
 import scipy.io
 import numpy as np
 from sklearn import preprocessing
-
+from common_primitives import utils
 from d3m import container
 from d3m.metadata import base as metadata_base
 from d3m.metadata import hyperparams
@@ -48,7 +47,8 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         'name': 'IPCMB feature selector',
         'keywords': ['rpi primitives'],
         'source': {
-            'name': 'RPI'
+            'name': 'RPI',
+            'contact': 'https://gitlab.com/Zijun-Cui/structured-fs',
         },
         'installation':[
             {
@@ -59,7 +59,7 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-                'version': '2.2.4'
+                'version': '2.2.6'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.IPCMBplus_Selector',
@@ -67,6 +67,9 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             metadata_base.PrimitiveAlgorithmType.MINIMUM_REDUNDANCY_FEATURE_SELECTION
         ],
         'primitive_family': metadata_base.PrimitiveFamily.FEATURE_SELECTION,
+        'preconditions': [
+            "NO_CATEGORICAL_VALUES",
+        ]
     })
 
 
@@ -78,10 +81,12 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        # convert categorical values to numerical values for training labels
-        LE = preprocessing.LabelEncoder()
-        LE = LE.fit(outputs.iloc[:,0])
-        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        # set training labels
+        [m,n] = inputs.shape
+        self._training_outputs = np.zeros((m,))
+        temp = list(outputs.iloc[:,0].as_matrix())
+        for i in np.arange(len(temp)):
+            self._training_outputs[i] = float(temp[i])
         
         # convert cateforical values to numerical values in training data
         metadata = inputs.metadata
@@ -118,7 +123,11 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
 
         a = rpi_feature_selection_toolbox.initialize()
         index = np.reshape(np.array(a.IPCMBplus()), [-1, ])
-        self._index = (index - 1).astype(int)
+        index = (index - 1).astype(int)
+        self._index = []
+        [m,] = index.shape
+        for ii in np.arange(m):
+            self._index.append(index[ii].item())
         self._fitted = True
 
         os.remove('rpi_data.mat')
@@ -129,7 +138,7 @@ class IPCMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:  # inputs: m x n numpy array
         if self._fitted:
             output = inputs.iloc[:, self._index]
-            output.metadata = inputs.metadata.clear(source=self, for_value=output, generate_metadata=True)
+            output.metadata = utils.select_columns_metadata(inputs.metadata, columns=self._index)
             return CallResult(output)
         else:
             raise ValueError('Model should be fitted first.')
@@ -152,7 +161,8 @@ class JMIplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         'name': 'JMIplus feature selector',
         'keywords': ['rpi primitives'],
         'source': {
-            'name': 'RPI'
+            'name': 'RPI',
+            'contact': 'https://gitlab.com/Zijun-Cui/structured-fs',
         },
         'installation':[
             {
@@ -163,12 +173,15 @@ class JMIplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.4'
+	            'version': '2.2.6'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.JMIplus_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
+	'preconditions': [
+		"NO_CATEGORICAL_VALUES",
+	]
     })
 
 
@@ -180,10 +193,12 @@ class JMIplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        # convert categorical values to numerical values for training labels
-        LE = preprocessing.LabelEncoder()
-        LE = LE.fit(outputs.iloc[:,0])
-        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        # set training labels
+        [m,n] = inputs.shape
+        self._training_outputs = np.zeros((m,))
+        temp = list(outputs.iloc[:,0].as_matrix())
+        for i in np.arange(len(temp)):
+            self._training_outputs[i] = float(temp[i])
         
         # convert cateforical values to numerical values in training data
         metadata = inputs.metadata
@@ -219,8 +234,11 @@ class JMIplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
 
         a = rpi_feature_selection_toolbox.initialize()
         index = np.reshape(np.array(a.JMIplus()), [-1, ])
-
-        self._index = (index - 1).astype(int)
+        index = (index - 1).astype(int)
+        self._index = []
+        [m,] = index.shape
+        for ii in np.arange(m):
+            self._index.append(index[ii].item())
         self._fitted = True
 
         os.remove('rpi_data.mat')
@@ -231,7 +249,7 @@ class JMIplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:  # inputs: m x n numpy array
         if self._fitted:
             output = inputs.iloc[:, self._index]
-            output.metadata = inputs.metadata.clear(source=self, for_value=output, generate_metadata=True)
+            output.metadata = utils.select_columns_metadata(inputs.metadata, columns=self._index)
             return CallResult(output)
         else:
             raise ValueError('Model should be fitted first.')
@@ -254,7 +272,8 @@ class STMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         'name': 'STMBplus feature selector',
         'keywords': ['rpi primitives'],
         'source': {
-            'name': 'RPI'
+            'name': 'RPI',
+            'contact': 'https://gitlab.com/Zijun-Cui/structured-fs',
         },
         'installation':[
             {
@@ -265,12 +284,15 @@ class STMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.4'
+	            'version': '2.2.6'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.STMBplus_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
+        'preconditions': [
+            "NO_CATEGORICAL_VALUES",
+        ]
     })
 
 
@@ -282,11 +304,13 @@ class STMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        # convert categorical values to numerical values for training labels
-        LE = preprocessing.LabelEncoder()
-        LE = LE.fit(outputs.iloc[:,0])
-        self._training_outputs = LE.transform(outputs.iloc[:,0])
-        
+        # set training labels
+        [m,n] = inputs.shape
+        self._training_outputs = np.zeros((m,))
+        temp = list(outputs.iloc[:,0].as_matrix())
+        for i in np.arange(len(temp)):
+            self._training_outputs[i] = float(temp[i])
+            
         # convert cateforical values to numerical values in training data
         metadata = inputs.metadata
         [m,n] = inputs.shape
@@ -321,8 +345,11 @@ class STMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
 
         a = rpi_feature_selection_toolbox.initialize()
         index = np.reshape(np.array(a.STMBplus()), [-1, ])
-        
-        self._index = (index - 1).astype(int)
+        index = (index - 1).astype(int)
+        self._index = []
+        [m,] = index.shape
+        for ii in np.arange(m):
+            self._index.append(index[ii].item())
         self._fitted = True
 
         os.remove('rpi_data.mat')
@@ -333,7 +360,7 @@ class STMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:  # inputs: m x n numpy array
         if self._fitted:
             output = inputs.iloc[:, self._index]
-            output.metadata = inputs.metadata.clear(source=self, for_value=output, generate_metadata=True)
+            output.metadata = utils.select_columns_metadata(inputs.metadata, columns=self._index)
             return CallResult(output)
         else:
             raise ValueError('Model should be fitted first.')
@@ -356,7 +383,8 @@ class aSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         'name': 'aSTMBplus feature selector',
         'keywords': ['rpi primitives'],
         'source': {
-            'name': 'RPI'
+            'name': 'RPI',
+            'contact': 'https://gitlab.com/Zijun-Cui/structured-fs',
         },
         'installation':[
             {
@@ -367,12 +395,15 @@ class aSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.4'
+	            'version': '2.2.6'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.aSTMBplus_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
+        'preconditions': [
+            "NO_CATEGORICAL_VALUES",
+        ]
     })
 
 
@@ -384,10 +415,12 @@ class aSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        # convert categorical values to numerical values for training labels
-        LE = preprocessing.LabelEncoder()
-        LE = LE.fit(outputs.iloc[:,0])
-        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        # set training labels
+        [m,n] = inputs.shape
+        self._training_outputs = np.zeros((m,))
+        temp = list(outputs.iloc[:,0].as_matrix())
+        for i in np.arange(len(temp)):
+            self._training_outputs[i] = float(temp[i])
         
         # convert cateforical values to numerical values in training data
         metadata = inputs.metadata
@@ -424,7 +457,11 @@ class aSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         a = rpi_feature_selection_toolbox.initialize()
         index = np.reshape(np.array(a.aSTMBplus()), [-1, ])
 
-        self._index = (index - 1).astype(int)
+        index = (index - 1).astype(int)
+        self._index = []
+        [m,] = index.shape
+        for ii in np.arange(m):
+            self._index.append(index[ii].item())
         self._fitted = True
 
         os.remove('rpi_data.mat')
@@ -435,7 +472,7 @@ class aSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:  # inputs: m x n numpy array
         if self._fitted:
             output = inputs.iloc[:, self._index]
-            output.metadata = inputs.metadata.clear(source=self, for_value=output, generate_metadata=True)
+            output.metadata = utils.select_columns_metadata(inputs.metadata, columns=self._index)
             return CallResult(output)
         else:
             raise ValueError('Model should be fitted first.')
@@ -458,7 +495,8 @@ class sSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         'name': 'sSTMBplus feature selector',
         'keywords': ['rpi primitives'],
         'source': {
-            'name': 'RPI'
+            'name': 'RPI',
+            'contact': 'https://gitlab.com/Zijun-Cui/structured-fs',
         },
         'installation':[
             {
@@ -469,12 +507,15 @@ class sSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.4'
+	            'version': '2.2.6'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.sSTMBplus_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
+        'preconditions': [
+            "NO_CATEGORICAL_VALUES",
+        ]
     })
 
 
@@ -486,10 +527,12 @@ class sSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        # convert categorical values to numerical values for training labels
-        LE = preprocessing.LabelEncoder()
-        LE = LE.fit(outputs.iloc[:,0])
-        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        # set training labels
+        [m,n] = inputs.shape
+        self._training_outputs = np.zeros((m,))
+        temp = list(outputs.iloc[:,0].as_matrix())
+        for i in np.arange(len(temp)):
+            self._training_outputs[i] = float(temp[i])
         
         # convert cateforical values to numerical values in training data
         metadata = inputs.metadata
@@ -525,7 +568,11 @@ class sSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
 
         a = rpi_feature_selection_toolbox.initialize()
         index = np.reshape(np.array(a.sSTMBplus()), [-1, ])
-        self._index = (index - 1).astype(int)
+        index = (index - 1).astype(int)
+        self._index = []
+        [m,] = index.shape
+        for ii in np.arange(m):
+            self._index.append(index[ii].item())
         self._fitted = True
 
         os.remove('rpi_data.mat')
@@ -536,7 +583,7 @@ class sSTMBplus_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params,
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:  # inputs: m x n numpy array
         if self._fitted:
             output = inputs.iloc[:, self._index]
-            output.metadata = inputs.metadata.clear(source=self, for_value=output, generate_metadata=True)
+            output.metadata = utils.select_columns_metadata(inputs.metadata, columns=self._index)
             return CallResult(output)
         else:
             raise ValueError('Model should be fitted first.')
@@ -558,7 +605,8 @@ class F_STMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hy
         'name': 'F_STMB feature selector',
         'keywords': ['rpi primitives'],
         'source': {
-            'name': 'RPI'
+            'name': 'RPI',
+            'contact': 'https://gitlab.com/Zijun-Cui/structured-fs',
         },
         'installation':[
             {
@@ -569,12 +617,15 @@ class F_STMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hy
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.4'
+	            'version': '2.2.6'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.F_STMB_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
+        'preconditions': [
+            "NO_CATEGORICAL_VALUES",
+        ]
     })
 
 
@@ -586,10 +637,12 @@ class F_STMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hy
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        # convert categorical values to numerical values for training labels
-        LE = preprocessing.LabelEncoder()
-        LE = LE.fit(outputs.iloc[:,0])
-        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        # set training labels
+        [m,n] = inputs.shape
+        self._training_outputs = np.zeros((m,))
+        temp = list(outputs.iloc[:,0].as_matrix())
+        for i in np.arange(len(temp)):
+            self._training_outputs[i] = float(temp[i])
         
         # convert cateforical values to numerical values in training data
         metadata = inputs.metadata
@@ -625,8 +678,11 @@ class F_STMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, Hy
 
         a = rpi_feature_selection_toolbox.initialize()
         index = np.reshape(np.array(a.F_STMB()), [-1, ])
- 
-        self._index = (index - 1).astype(int)
+        index = (index - 1).astype(int)
+        self._index = []
+        [m,] = index.shape
+        for ii in np.arange(m):
+            self._index.append(index[ii].item())
         self._fitted = True
 
         os.remove('rpi_data.mat')
@@ -661,7 +717,8 @@ class F_aSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         'name': 'F_aSTMB feature selector',
         'keywords': ['rpi primitives'],
         'source': {
-            'name': 'RPI'
+            'name': 'RPI',
+            'contact': 'https://gitlab.com/Zijun-Cui/structured-fs',
         },
         'installation':[
             {
@@ -672,12 +729,15 @@ class F_aSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.4'
+	            'version': '2.2.6'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.F_aSTMB_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
+        'preconditions': [
+            "NO_CATEGORICAL_VALUES",
+        ]
     })
 
 
@@ -689,10 +749,12 @@ class F_aSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-        # convert categorical values to numerical values for training labels
-        LE = preprocessing.LabelEncoder()
-        LE = LE.fit(outputs.iloc[:,0])
-        self._training_outputs = LE.transform(outputs.iloc[:,0])
+        # set training labels
+        [m,n] = inputs.shape
+        self._training_outputs = np.zeros((m,))
+        temp = list(outputs.iloc[:,0].as_matrix())
+        for i in np.arange(len(temp)):
+            self._training_outputs[i] = float(temp[i])
         
         # convert cateforical values to numerical values in training data
         metadata = inputs.metadata
@@ -728,8 +790,11 @@ class F_aSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
 
         a = rpi_feature_selection_toolbox.initialize()
         index = np.reshape(np.array(a.F_aSTMB()), [-1, ])
-
-        self._index = (index - 1).astype(int)
+        index = (index - 1).astype(int)
+        self._index = []
+        [m,] = index.shape
+        for ii in np.arange(m):
+            self._index.append(index[ii].item())
         self._fitted = True
 
         os.remove('rpi_data.mat')
@@ -740,7 +805,7 @@ class F_aSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:  # inputs: m x n numpy array
         if self._fitted:
             output = inputs.iloc[:, self._index]
-            output.metadata = inputs.metadata.clear(source=self, for_value=output, generate_metadata=True)
+            output.metadata = utils.select_columns_metadata(inputs.metadata, columns=self._index)
             return CallResult(output)
         else:
             raise ValueError('Model should be fitted first.')
@@ -763,7 +828,8 @@ class F_sSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         'name': 'F_sSTMB feature selector',
         'keywords': ['rpi primitives'],
         'source': {
-            'name': 'RPI'
+            'name': 'RPI',
+            'contact': 'https://gitlab.com/Zijun-Cui/structured-fs',
         },
         'installation':[
             {
@@ -774,12 +840,15 @@ class F_sSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
             {
                 'type': metadata_base.PrimitiveInstallationType.PIP,
                 'package': 'rpi_featureSelection_python_tools',
-	            'version': '2.2.4'
+	            'version': '2.2.6'
             }
         ],
         'python_path': 'd3m.primitives.rpi_featureSelection_python_tools.F_sSTMB_Selector',
         'algorithm_types': ['MINIMUM_REDUNDANCY_FEATURE_SELECTION'],
         'primitive_family': 'FEATURE_SELECTION',
+        'preconditions': [
+            "NO_CATEGORICAL_VALUES",
+        ]
     })
 
 
@@ -791,10 +860,12 @@ class F_sSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         self._fitted = False
 
     def set_training_data(self, *, inputs: Inputs, outputs: Outputs) -> None:
-       # convert categorical values to numerical values for training labels
-        LE = preprocessing.LabelEncoder()
-        LE = LE.fit(outputs.iloc[:,0])
-        self._training_outputs = LE.transform(outputs.iloc[:,0])
+       # set training labels
+        [m,n] = inputs.shape
+        self._training_outputs = np.zeros((m,))
+        temp = list(outputs.iloc[:,0].as_matrix())
+        for i in np.arange(len(temp)):
+            self._training_outputs[i] = float(temp[i])
         
         # convert cateforical values to numerical values in training data
         metadata = inputs.metadata
@@ -831,7 +902,11 @@ class F_sSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
         a = rpi_feature_selection_toolbox.initialize()
         index = np.reshape(np.array(a.F_sSTMB()), [-1, ])
 
-        self._index = (index - 1).astype(int)
+        index = (index - 1).astype(int)
+        self._index = []
+        [m,] = index.shape
+        for ii in np.arange(m):
+            self._index.append(index[ii].item())
         self._fitted = True
 
         os.remove('rpi_data.mat')
@@ -842,7 +917,7 @@ class F_sSTMB_Selector(SupervisedLearnerPrimitiveBase[Inputs, Outputs, Params, H
     def produce(self, *, inputs: Inputs, timeout: float = None, iterations: int = None) -> base.CallResult[Outputs]:  # inputs: m x n numpy array
         if self._fitted:
             output = inputs.iloc[:, self._index]
-            output.metadata = inputs.metadata.clear(source=self, for_value=output, generate_metadata=True)
+            output.metadata = utils.select_columns_metadata(inputs.metadata, columns=self._index)
             return CallResult(output)
         else:
             raise ValueError('Model should be fitted first.')
